package ca.recrown.islandsurvivalcraft.interaction.commands.runnables;

import java.util.HashSet;
import java.util.Set;

import org.bukkit.Bukkit;
import org.bukkit.Color;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;
import ca.recrown.islandsurvivalcraft.utilities.GeneralUtilities;
import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.world.WorldInfo;
import ca.recrown.islandsurvivalcraft.world.Information.IslandInformation;

public class IslandCommand implements CommandRunnable {
    private IslandSurvivalCraft islandSurvivalCraft;
    private final Particle.DustOptions EDGE_DUST_OPTIONS = new Particle.DustOptions(Color.RED, 1f);
    private final Particle.DustOptions ORIGIN_DUST_OPTIONS = new Particle.DustOptions(Color.LIME, 1.5f);
    private final int PARTICLE_AMOUNT = 1;
    private final double OFFSET_X = 0.0d, OFFSET_Y = 6.0d, OFFSET_Z = 0.0d;
    private final double EXTRA = 0d;
    HashSet<Player> playersHighlighting = new HashSet<>();

    @Override
    public String getDescription() {
        return "Various debugging information about islands.";
    }

    @Override
    public boolean invoke(CommandSender sender, String[] args) {
        if (!(sender instanceof Player)) {
            sender.sendMessage("This command can only be run as a player!");
            return false;
        }
        Player player = (Player) sender;
        if (args.length == 0 || args[0].isEmpty()) {
            return false;
        }
        if (args[0].toLowerCase().equals("highlight")) {
            if (playersHighlighting.add(player)) {
                sender.sendMessage("You are now highlighting islands.");
            } else {
                sender.sendMessage("You are already highlighting islands.");
            }
        } else if (args[0].toLowerCase().equals("stop")) {
            if (playersHighlighting.remove(player)) {
                sender.sendMessage("You are no longer highlighting islands.");
            } else {
                sender.sendMessage("You weren't highlighting islands.");
            }
        } else if (args[0].toLowerCase().equals("origin")) {
            Point2 playerLoc = new Point2(player.getLocation().getBlockX(), player.getLocation().getBlockZ());
            sender.sendMessage("Current island's origin point: " + islandSurvivalCraft.getWorldInfoManager().retrieve(player.getWorld().getName()).getIslandInfoManager().getIslandInformation(playerLoc, true).getIslandOrigin());
        } else if (args[0].toLowerCase().equals("size")) {
            Point2 playerLoc = new Point2(player.getLocation().getBlockX(), player.getLocation().getBlockZ());
            IslandInformation info = islandSurvivalCraft.getWorldInfoManager().retrieve(player.getWorld().getName()).getIslandInfoManager().getIslandInformation(playerLoc, true);
            if (info != null) {
                sender.sendMessage("Current island's raw size in blocks: " + info.getIslandSize());
            } else {
                sender.sendMessage("Current location is not an island!");
            }
        } else {
            return false;
        }
        return true;
    }

    @Override
    public void initialize(IslandSurvivalCraft islandsurvivalcraft) {
        this.islandSurvivalCraft = islandsurvivalcraft;
        Bukkit.getScheduler().scheduleSyncRepeatingTask(islandsurvivalcraft, new Runnable() {
            @Override
            public void run() {
                for (Player player : playersHighlighting) {
                    if (player == null) return;
                    if (!player.isOnline()) {
                        playersHighlighting.remove(player);
                        return;
                    }
                    World world = player.getWorld();
                    WorldInfo worldInfo = islandsurvivalcraft.getWorldInfoManager().retrieve(world.getName());
                    Location playerLocation = player.getLocation();
                    double playerX = playerLocation.getX(), playerY = playerLocation.getY(), playerZ = playerLocation.getZ();
                    Point2 playerCoords = new Point2(playerLocation.getBlockX(), playerLocation.getBlockZ());
                    IslandInformation islandInfo = worldInfo.getIslandInfoManager().getIslandInformation(playerCoords, true);
                    if (islandInfo != null) {
                        Set<Point2> islandBorder = islandInfo.getEdgeCoordinates();
                        for (Point2 current : islandBorder) {
                            spawnParticle(false, true, world, current.x, playerY, current.y, EDGE_DUST_OPTIONS);
                        }
                        Point2 islandOrigin = islandInfo.getIslandOrigin();
                        double xDirection = GeneralUtilities.addMagnitude(Math.max(Math.min(1, islandOrigin.x - playerCoords.x), -1), 1) + playerX;
                        double yDirection = GeneralUtilities.addMagnitude(Math.max(Math.min(1, islandOrigin.y - playerCoords.y), -1), 1) + playerZ;
                        spawnParticle(false, false, world, xDirection, playerY + 1, yDirection, ORIGIN_DUST_OPTIONS);
                        spawnParticle(true, true, world, islandOrigin.x, playerY, islandOrigin.y, ORIGIN_DUST_OPTIONS);
                    }
                }
            }
        }, 0, 10);

    }

    private void spawnParticle(boolean focus, boolean column, World world, double x, double y, double z, Particle.DustOptions dustOptions) {
        world.spawnParticle(Particle.REDSTONE, x, y, z, focus ? PARTICLE_AMOUNT * 2 : PARTICLE_AMOUNT, OFFSET_X, column ? OFFSET_Y : 0.2, OFFSET_Z, EXTRA, dustOptions, false);
    }

    @Override
    public String getDetailedDescription() {
        return "Usage: \"island <arg>\". replacing <args> with \"highlight\"  will start highlighting the island by " +
        "displaying red particles that indicate the islands border, and green particles indicate the origin. \"stop\" will stop the highlighting. " +
        "\"origin\" will print the current island's origin point. " +
        "\"size\" will print the raw island size in blocks including the shallow portions.";
    }

    @Override
    public boolean requiresInitialization() {
        return islandSurvivalCraft == null;
    }
}