package ca.recrown.islandsurvivalcraft.utilities.pathfinding;

import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;

public class Node extends Point2 implements Comparable<Point2> {
    private final NodeComparable nodeComparable;
    private final Point2 parent;
    private final Point2 goal;
    private Point2 child;

    public Node(Point2 parent, int x, int y, Point2 goal, NodeComparable nodeComparable) {
        super(x, y);
        this.parent = parent;
        this.goal = goal;
        this.nodeComparable = nodeComparable;
    }

    public Node(Point2 parent, Point2 location, Point2 goal, NodeComparable nodeComparable) {
        this(parent, location.x, location.y, goal, nodeComparable);
    }

    /**
     * Sets the child of this node. Can only be set once.
     * @param child the child to set.
     */
    public void setChild(Point2 child) {
        if (this.child != null) throw new IllegalStateException("Child has already been set."); 
        this.child = child;
    }

    @Override
    public int compareTo(Point2 o) {
        if (nodeComparable != null) return nodeComparable.compare(this, o);
        if (goal == null) return 0;
        double distDiff = distance(goal) - distance(o);
        if (distDiff == 0) {
            return 0;
        } else if (distDiff < 0) {
            return -1;
        } else {
            return 1;
        }
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }

    /**
     * @return the parent
     */
    public Point2 getParent() {
        return parent;
    }
}