package ca.recrown.islandsurvivalcraft.world.Information;

import java.util.HashSet;

import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;

public class IslandInformationBuilder {
    private final HashSet<Point2> islandCoordinates = new HashSet<>();
    private final HashSet<Point2> edgeCoordinates = new HashSet<>();
    private final HashSet<Point2> chunksUsed = new HashSet<>();
    private boolean built = false;

    public boolean addCoordinate(Point2 coordinate, boolean isEdge) {
        if (coordinate == null) throw new NullPointerException("Coordinate cannot be null!");
        if (built) throw new IllegalStateException("Island information has already been built!");
        if (isEdge) edgeCoordinates.add(coordinate);
        return islandCoordinates.add(coordinate);
    }

    public boolean addChunk(Point2 chunkCoordinate) {
        if (chunkCoordinate == null) throw new NullPointerException("Chunk coordinate cannot be null!");
        if (built) throw new IllegalStateException("Island information has already been built!");
        return chunksUsed.add(chunkCoordinate);
    }

    public IslandInformation build(Point2 islandOrigin) {
        if (islandOrigin == null) throw new NullPointerException("IslandOrigin cannot be null.");
        if (built) throw new IllegalStateException("This island informationh as already been built!");
        built = true;
        return new IslandInformation(islandOrigin, islandCoordinates, edgeCoordinates, chunksUsed);
    }

    /**
     * @return the chunksUsed
     */
    public HashSet<Point2> getChunksUsed() {
        return chunksUsed;
    }
}